@@ -2,7 +2,7 @@
  * File name: envelope.h
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor <http://iuriepage.wordpress.com>
+ * Copyright (C) 2017 Iurie Nistor 
  *
  * This file is part of Geonkick.
  *
@@ -43,6 +43,8 @@ Envelope::Envelope(const RkRect &area)
         , pointSelected{false}
         , envelopeCategory{Category::Oscillator1}
         , envelopeType{Type::Amplitude}
+        , editedPointIndex{0}
+        , isEditingPoint{false}
 {
 }
 
@@ -95,7 +97,7 @@ void Envelope::drawTimeScale(RkPainter &painter)
         font.setSize(10);
         painter.setFont(font);
 
-        auto val = envelopeLengh() / 10;
+        auto val = envelopeLength() / 10;
         int dx = W() / 10;
         RkPoint point = getOrigin();
         int x  = point.x() + dx;
@@ -115,7 +117,7 @@ void Envelope::drawTimeScale(RkPainter &painter)
         painter.setFont(font);
         painter.setPen(RkPen(RkColor(180, 180, 180, 200)));
         painter.drawText(point.x() + W() / 2 - 35, point.y() +  font.size() + 10,
-                         "Length, " + std::to_string(std::llround(envelopeLengh())) + " ms");
+                         "Length, " + std::to_string(std::llround(envelopeLength())) + " ms");
 }
 
 void Envelope::drawValueScale(RkPainter &painter)
@@ -266,29 +268,26 @@ void Envelope::drawPointValue(RkPainter &painter, const RkPoint &point, double v
             || type() == Type::DistortionDrive
             || type() == Type::DistortionVolume) {
                 value *= envelopeAmplitude();
-                std::ostringstream ss;
 		if (type() == Type::DistortionDrive || type() == Type::DistortionVolume)
-			ss << std::setprecision(2) << value * pow(10, 36.0 / 20);
-		else
-			ss << std::setprecision(2) << value;
-                painter.drawText(point.x(), point.y(), ss.str());
+			value *= pow(10, 36.0 / 20);
+                painter.drawText(point.x(), point.y(), Geonkick::doubleToStr(value, 2));
         } else if (type() == Type::PitchShift) {
                 value *= envelopeAmplitude();
-                std::ostringstream ss;
-                ss << std::fixed << std::setprecision(1) << 2 * value - envelopeAmplitude();
-                painter.drawText(point.x(), point.y(), ss.str());
+                value = 2 * value - envelopeAmplitude();
+                painter.drawText(point.x(), point.y(), Geonkick::doubleToStr(value, 1));
         } else if (type() == Envelope::Type::Frequency || type() == Type::FilterCutOff) {
                 value *= envelopeAmplitude();
                 if (value < 20)
                         painter.drawText(point.x(), point.y(), "20Hz " + frequencyToNote(20));
                 if (value >= 20 && value < 1000) {
-                        painter.drawText(point.x(), point.y(), std::to_string(std::llround(value))
+                        painter.drawText(point.x(), point.y(),
+                                         Geonkick::doubleToStr(value, 0)
                                          + "Hz " + frequencyToNote(value));
                 } else if (value >= 1000 && value <= 20000) {
-                        std::ostringstream ss;
-                        ss.precision(1);
-                        ss << std::fixed << value / 1000;
-                        painter.drawText(point.x(), point.y(), ss.str() + "kHz " + frequencyToNote(value));
+                        painter.drawText(point.x(), point.y(),
+                                         Geonkick::doubleToStr(value / 1000, 0)
+                                         + std::string("kHz ")
+                                         + frequencyToNote(value));
                 }
         }
 }
@@ -349,6 +348,44 @@ void Envelope::selectPoint(const RkPoint &point)
         }
 }
 
+void Envelope::updateSelectedPointValue(double val)
+{
+        if (hasEditingPoint() && editedPointIndex < envelopePoints.size()) {
+                pointUpdatedEvent(editedPointIndex,
+                                  envelopePoints[editedPointIndex].x(),
+                                  convertFromHumanValue(val));
+                updatePoints();
+        }        
+}
+
+void Envelope::setEditCurrentPoint(bool edit)
+{
+        if (hasOverPoint() && overPointIndex < envelopePoints.size()) {
+                editedPointIndex = overPointIndex;
+                isEditingPoint = edit;
+                return;
+        }
+        editedPointIndex = 0;
+        isEditingPoint = false;
+}
+
+double Envelope::getSelectedPointValue() const
+{
+        return convertToHumanValue(getSelectedPoint().y());
+}
+
+RkRealPoint Envelope::getSelectedPoint() const
+{
+        if (hasSelected()) {
+                if (overPointIndex < envelopePoints.size())
+                        return envelopePoints[selectedPointIndex];
+        } else if (hasOverPoint()) {
+                if (overPointIndex < envelopePoints.size())
+                        return envelopePoints[overPointIndex];
+        }
+        return {};
+}
+
 void Envelope::unselectPoint(void)
 {
         pointSelected = false;
@@ -582,9 +619,13 @@ void Envelope::setDotRadius(int radius)
         dotRadius = radius;
 }
 
-std::string Envelope::frequencyToNote(rk_real f)
+/**
+ * This conversion should comply with this table:
+ * https://homes.luddy.indiana.edu/donbyrd/Teach/MusicalPitchesTable.htm
+ */
+std::string Envelope::frequencyToNote(rk_real f) const
 {
-        if (f < 27.50 || f > 13289.75)
+        if (f < 27.500 || f > 13289.752)
                 return "";
         constexpr std::array<const char*, 12> noteNames = {"C",
                                                            "C#",
@@ -598,9 +639,73 @@ std::string Envelope::frequencyToNote(rk_real f)
                                                            "A",
                                                            "A#",
                                                            "B"};
-        int midiNote  = 12 * (log2(f / 27.5)) + 21;
+        int midiNote  = std::round(12.0 * (log2(f / 27.5)) + 21.0);
         int octave    = midiNote / 12 - 1;
-        int noteIndex = midiNote % 12;
+        int noteIndex = static_cast<int>(midiNote) % 12;
         return noteNames[noteIndex] + std::to_string(octave);
 }
 
+double Envelope::convertToHumanValue(double val) const
+{
+        if (type() == Envelope::Type::Amplitude
+            || type() == Type::DistortionDrive
+            || type() == Type::DistortionVolume) {
+                val *= envelopeAmplitude();
+		if (type() == Type::DistortionDrive || type() == Type::DistortionVolume)
+			return val * pow(10, 36.0 / 20);
+		else
+			return val;
+        } else if (type() == Type::PitchShift) {
+                val *= envelopeAmplitude();
+                return 2 * val - envelopeAmplitude();
+        } else if (type() == Envelope::Type::Frequency || type() == Type::FilterCutOff) {
+                val *= envelopeAmplitude();
+        }
+        return val;
+}
+
+double Envelope::convertFromHumanValue(double val) const
+{
+        if ( envelopeAmplitude() == 0.0 )
+                return val;
+        
+        if (type() == Envelope::Type::Amplitude
+            || type() == Type::DistortionDrive
+            || type() == Type::DistortionVolume) {
+                val /= envelopeAmplitude();
+		if (type() == Type::DistortionDrive || type() == Type::DistortionVolume)
+			val /= pow(10, 36.0 / 20);
+        } else if (type() == Type::PitchShift) {
+                val = (val / envelopeAmplitude() + 1.0) / 2.0;
+        } else if (type() == Envelope::Type::Frequency || type() == Type::FilterCutOff) {
+                val /= envelopeAmplitude();
+        }
+
+        return std::clamp(val, 0.0, 1.0);        
+}
+
+bool Envelope::hasEditingPoint() const
+{
+        return isEditingPoint;
+}
+
+std::string Envelope::getCurrentPointInfo() const
+{
+        if (!hasSelected() && !hasOverPoint() )
+                return std::string();
+
+        auto point = getSelectedPoint();
+        double roundedValue = std::round(convertToHumanValue(point.y()) * 10000.0) / 10000.0;
+        std::string info = Geonkick::doubleToStr(roundedValue, 4);
+        switch (type()) {
+        case Type::Frequency:
+        case Type::FilterCutOff:
+                info += "Hz " + frequencyToNote(point.y());
+                break;
+        default:
+                break;
+        }
+
+        info += ", " + Geonkick::doubleToStr(point.x() * envelopeLength(), 0) + "ms";
+        return info;
+}
