@@ -2,7 +2,7 @@
  * File name: export_widget.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2018 Iurie Nistor (http://iuriepage.wordpress.com)
+ * Copyright (C) 2018 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -76,6 +76,10 @@ ExportWidget::ExportWidget(GeonkickWidget *parent, GeonkickApi *api)
         , selectedFormat{ExportFormat::Wav16}
         , channelsType{ChannelsType::Mono}
 {
+        setPosition(30, 40);
+        setBorderWidth(1);
+        setBorderColor(40, 40, 40);
+
         std::string format = geonkickApi->getSettings("ExportDialog/Format");
         if (!format.empty())
                 selectedFormat = static_cast<ExportFormat>(std::stoi(format));
@@ -261,6 +265,7 @@ void ExportWidget::browse()
         fileNameEdit->setFocus(false);
         auto fileDialog = new FileDialog(this, FileDialog::Type::Open,
                                          "Select Path - " + std::string(GEONKICK_NAME));
+        fileDialog->setPosition(20, 20);
         fileDialog->setHomeDirectory(geonkickApi->getSettings("GEONKICK_CONFIG/HOME_PATH"));
         fileDialog->setCurrentDirectoy(geonkickApi->currentWorkingPath("ExportDialog/Location"));
         RK_ACT_BIND(fileDialog, selectedFile,
