@@ -2,7 +2,7 @@
  * File name: geonkick_api.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor <http://iuriepage.wordpress.com>
+ * Copyright (C) 2017 Iurie Nistor 
  *
  * This file is part of Geonkick.
  *
@@ -34,8 +34,9 @@
 
 #include <sndfile.h>
 
-GeonkickApi::GeonkickApi(int sample_rate)
-        : geonkickApi{nullptr}
+GeonkickApi::GeonkickApi(int sample_rate, InstanceType instance, geonkick *dsp)
+        : geonkickApi{dsp}
+        , instanceType{instance}
         , limiterLevelers{}
         , jackEnabled{false}
         , standaloneInstance{false}
@@ -57,6 +58,16 @@ GeonkickApi::~GeonkickApi()
                 geonkick_free(&geonkickApi);
 }
 
+void GeonkickApi::setInstanceType(InstanceType type)
+{
+        instanceType = type;
+}
+
+GeonkickApi::InstanceType GeonkickApi::getInstanceType() const
+{
+        return instanceType;
+}
+
 unsigned int GeonkickApi::getVersion()
 {
         return GEONKICK_VERSION;
@@ -68,13 +79,23 @@ void GeonkickApi::setEventQueue(RkEventQueue *queue)
         eventQueue = queue;
 }
 
+bool GeonkickApi::initDSP()
+{
+          if (!geonkickApi) {
+                  if (geonkick_create(&geonkickApi, sampleRate) != GEONKICK_OK) {
+	                  GEONKICK_LOG_ERROR("can't create geonkick API");
+                          return false;
+                  }
+	  }
+	  return true;
+}
+
 bool GeonkickApi::init()
 {
-        loadPresets();
-  	if (geonkick_create(&geonkickApi, sampleRate) != GEONKICK_OK) {
-	        GEONKICK_LOG_ERROR("can't create geonkick API");
-                return false;
-  	}
+        if (!initDSP())
+	        return false;
+
+	loadPresets();
         jackEnabled = geonkick_is_module_enabed(geonkickApi, GEONKICK_MODULE_JACK);
 	geonkick_enable_synthesis(geonkickApi, false);
 
@@ -98,7 +119,7 @@ bool GeonkickApi::init()
         return true;
 }
 
-size_t GeonkickApi::numberOfChannels() const
+size_t GeonkickApi::numberOfChannels()
 {
         return geonkick_channels_number();
 }
