@@ -2,7 +2,7 @@
  * File name: geonkick_api.h
  * Project: Geonkick (A percussion synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor <http://iuriepage.wordpress.com>
+ * Copyright (C) 2017 Iurie Nistor 
  *
  * This file is part of Geonkick.
  *
@@ -39,6 +39,12 @@ class GeonkickApi : public RkObject {
 
  public:
 
+  enum class InstanceType: int {
+                             Standalone = 0,
+                             Lv2        = 1,
+                             Vst3       = 2
+  };
+
   enum class Layer: int {
                 Layer1 = 0,
                 Layer2 = 1,
@@ -51,7 +57,7 @@ class GeonkickApi : public RkObject {
          Noise       = 2
   };
 
-  enum class FunctionType:int {
+  enum class FunctionType: int {
           Sine          = GEONKICK_OSC_FUNC_SINE,
           Square        = GEONKICK_OSC_FUNC_SQUARE,
           Triangle      = GEONKICK_OSC_FUNC_TRIANGLE,
@@ -62,7 +68,7 @@ class GeonkickApi : public RkObject {
           Sample        = GEONKICK_OSC_FUNC_SAMPLE,
   };
 
-  enum class EnvelopeType:int {
+  enum class EnvelopeType: int {
           Amplitude = GEONKICK_AMPLITUDE_ENVELOPE,
           Frequency = GEONKICK_FREQUENCY_ENVELOPE,
 	  FilterCutOff = GEONKICK_FILTER_CUTOFF_ENVELOPE,
@@ -71,17 +77,22 @@ class GeonkickApi : public RkObject {
           PitchShift = GEONKICK_PITCH_SHIFT_ENVELOPE
   };
 
-  enum class FilterType:int {
+  enum class FilterType: int {
           LowPass  = GEONKICK_FILTER_LOW_PASS,
           HighPass = GEONKICK_FILTER_HIGH_PASS,
           BandPass = GEONKICK_FILTER_BAND_PASS
   };
 
-  GeonkickApi(int sample_rate = Geonkick::defaultSampleRate);
+        GeonkickApi(int sample_rate = Geonkick::defaultSampleRate,
+                    InstanceType instance = InstanceType::Standalone,
+                    geonkick *dsp = nullptr);
   ~GeonkickApi();
+  void setInstanceType(InstanceType type);
+  InstanceType getInstanceType() const;
   static unsigned int getVersion();
-  size_t numberOfChannels() const;
+  size_t static numberOfChannels();
   void setEventQueue(RkEventQueue *queue);
+  bool initDSP();
   bool init();
   void registerCallbacks(bool b);
   std::vector<std::unique_ptr<Oscillator>> oscillators(void);
@@ -341,6 +352,7 @@ protected:
 
 private:
   mutable struct geonkick *geonkickApi;
+  InstanceType instanceType;
   std::array<std::atomic<double>, GEONKICK_MAX_PERCUSSIONS> limiterLevelers;
   bool jackEnabled;
   bool standaloneInstance;
