@@ -2,7 +2,7 @@
  * File name: geonkick_button.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2018 Iurie Nistor (http://iuriepage.wordpress.com)
+ * Copyright (C) 2018 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -31,10 +31,10 @@ GeonkickButton::GeonkickButton(GeonkickWidget *parent)
 
 void GeonkickButton::setPressedImage(const RkImage &img)
 {
-        setImage(img, RkButton::ButtonImage::ImagePressed);
+        setImage(img, RkButton::State::Pressed);
 }
 
 void GeonkickButton::setUnpressedImage(const RkImage &img)
 {
-        setImage(img, RkButton::ButtonImage::ImageUnpressed);
+        setImage(img, RkButton::State::Unpressed);
 }
