@@ -2,7 +2,7 @@
  * File name: globals.h
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor <http://iuriepage.wordpress.com>
+ * Copyright (C) 2017 Iurie Nistor 
  *
  * This file is part of Geonkick.
  *
@@ -42,16 +42,28 @@
 #include <mutex>
 #include <thread>
 #include <iomanip>
+#include <math.h>
 
 #include <RkObject.h>
 
 #ifdef RK_LOG_DEBUG_LEVEL
-#define GEONKICK_LOG_INFO(msg) std::cout << __PRETTY_FUNCTION__ << "[INFO] " << msg << std::endl;
-#define GEONKICK_LOG_ERROR(msg) std::cout << __PRETTY_FUNCTION__ << "[ERROR] " << msg << std::endl;
-#define GEONKICK_LOG_DEBUG(msg) std::cout << __PRETTY_FUNCTION__ << "[DEBUG] " << msg << std::endl;
+#define GEONKICK_LOG_INFO(msg) std::cout << "[" << GEONKICK_NAME << "] " \
+                               << " [" << std::this_thread::get_id() << "] " \
+                               << __PRETTY_FUNCTION__ << "[INFO] " \
+                               << msg << std::endl;
+#define GEONKICK_LOG_ERROR(msg) std::cout << "[" << GEONKICK_NAME << "] " \
+                                << " [" << std::this_thread::get_id() << "] " \
+                                << __PRETTY_FUNCTION__ << "[ERROR] " \
+                                << msg << std::endl;
+#define GEONKICK_LOG_DEBUG(msg) std::cout << "[" << GEONKICK_NAME << "] " \
+                                << " [" << std::this_thread::get_id() << "] " \
+                                << __PRETTY_FUNCTION__ << "[DEBUG] " \
+                                << msg << std::endl;
 #else
-#define GEONKICK_LOG_INFO(msg) std::cout << "[INFO] " << msg << std::endl;
-#define GEONKICK_LOG_ERROR(msg) std::cout << "[ERROR] " << msg << std::endl;
+#define GEONKICK_LOG_INFO(msg) std::cout << "[" << GEONKICK_NAME << "] " \
+                               << "[INFO] " << msg << std::endl;
+#define GEONKICK_LOG_ERROR(msg) std::cout << "[" << GEONKICK_NAME << "] " \
+                               << "[ERROR] " << msg << std::endl;
 #define GEONKICK_LOG_DEBUG(msg)
 #endif //  // GKICK_LIB_LOG_LEVEL_DEBUG
 
@@ -61,17 +73,33 @@ namespace Geonkick
         constexpr char appName[] = GEONKICK_NAME;
         constexpr char applicationName[] = GEONKICK_APP_NAME;
         constexpr char applicationVersionStr[] = GEOKICK_VERSION_STRING;
-        inline  RkString geonkickInfo()
+        constexpr double toDecibel(double val)
         {
-                RkString instanceType("Standalone");
-                if (geonkick_is_plugin()) {
-#ifdef GEONKICK_SINGLE
-                        instanceType = "LV2, Single";
-#else
-                        instanceType = "LV2, Multi";
-#endif // GEONKICK_SINGLE
-                }
-                return instanceType + ", v" + RkString(Geonkick::applicationVersionStr);
+                if (val < std::numeric_limits<double>::min())
+                        return -80;
+                return 20 * log10(val);
+        }
+
+        constexpr double fromDecibel(double decibel)
+        {
+                if (decibel < -80)
+                        return 0;
+                return pow(10,  decibel / 20);
+        }
+
+        constexpr double truncateDouble(double val, unsigned int precision)
+        {
+                double n = std::pow(10, precision);
+                return std::trunc(val * n) / n;
+        }
+
+        inline std::string doubleToStr(double val, unsigned int precision)
+        {
+                std::stringstream ss;
+                ss << std::fixed
+                   << std::setprecision(precision)
+                   << truncateDouble(val, precision);
+                return ss.str();
         }
 
 } // namespace Geonkick
