@@ -2,7 +2,7 @@
  * File name: oscillator_envelope.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor <(http://iuriepage.wordpress.com>
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -57,7 +57,7 @@ void OscillatorEnvelope::pointRemovedEvent(unsigned int index)
 
 }
 
-double OscillatorEnvelope::envelopeLengh(void) const
+double OscillatorEnvelope::envelopeLength(void) const
 {
         return oscillator->envelopeLength();
 }
