@@ -2,7 +2,7 @@
  * File name: oscillator_envelope.h
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor (http://iuriepage.wordpress.com)
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -33,7 +33,7 @@ class OscillatorEnvelope: public Envelope
  public:
 
   OscillatorEnvelope(Oscillator* osc, const RkRect &area);
-  double envelopeLengh(void) const;
+  double envelopeLength(void) const;
   Oscillator* getOscillator() const;
 
  protected:
