@@ -2,7 +2,7 @@
  * File name: oscillator_group_box.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor (http://iuriepage.wordpress.com)
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -29,16 +29,21 @@
 #include "geonkick_slider.h"
 #include "filter.h"
 #include "file_dialog.h"
+#include "ViewState.h"
 
 #include <RkLabel.h>
 
-RK_DECLARE_IMAGE_RC(osc1_radio_button);
-RK_DECLARE_IMAGE_RC(osc1_radio_button_active);
-RK_DECLARE_IMAGE_RC(osc2_radio_button);
-RK_DECLARE_IMAGE_RC(osc2_radio_button_active);
-RK_DECLARE_IMAGE_RC(noise_radio_button);
-RK_DECLARE_IMAGE_RC(noise_radio_button_active);
+RK_DECLARE_IMAGE_RC(controls_osc1_on);
+RK_DECLARE_IMAGE_RC(controls_osc1_hover);
+RK_DECLARE_IMAGE_RC(controls_osc1_off);
+RK_DECLARE_IMAGE_RC(controls_osc2_on);
+RK_DECLARE_IMAGE_RC(controls_osc2_hover);
+RK_DECLARE_IMAGE_RC(controls_osc2_off);
+RK_DECLARE_IMAGE_RC(controls_noise_on);
+RK_DECLARE_IMAGE_RC(controls_noise_hover);
+RK_DECLARE_IMAGE_RC(controls_noise_off);
 RK_DECLARE_IMAGE_RC(fm_radio);
+RK_DECLARE_IMAGE_RC(fm_radio_hover);
 RK_DECLARE_IMAGE_RC(fm_radio_active);
 RK_DECLARE_IMAGE_RC(wf_bk_hbox);
 RK_DECLARE_IMAGE_RC(wave_button_sine);
@@ -61,59 +66,103 @@ RK_DECLARE_IMAGE_RC(noise_type_white_active);
 RK_DECLARE_IMAGE_RC(noise_type_brownian);
 RK_DECLARE_IMAGE_RC(noise_type_brownian_active);
 RK_DECLARE_IMAGE_RC(knob_bk_image);
+RK_DECLARE_IMAGE_RC(osc_ampl_button_off);
+RK_DECLARE_IMAGE_RC(osc_ampl_button_on);
+RK_DECLARE_IMAGE_RC(osc_ampl_button_hover);
+RK_DECLARE_IMAGE_RC(osc_freq_button_off);
+RK_DECLARE_IMAGE_RC(osc_freq_button_on);
+RK_DECLARE_IMAGE_RC(osc_freq_button_hover);
+RK_DECLARE_IMAGE_RC(osc_pitch_button_off);
+RK_DECLARE_IMAGE_RC(osc_pitch_button_on);
+RK_DECLARE_IMAGE_RC(osc_pitch_button_hover);
 
 OscillatorGroupBox::OscillatorGroupBox(GeonkickWidget *parent, Oscillator *osc)
           : GeonkickGroupBox{parent}
-           , oscillator{osc}
-           , oscillatorCheckbox{nullptr}
-           , fmCheckbox{nullptr}
-           , filterBox{nullptr}
-           , sineButton{nullptr}
-           , squareButton{nullptr}
-           , triangleButton{nullptr}
-           , sawtoothButton{nullptr}
-           , sampleButton{nullptr}
-           , sampleBrowseButton{nullptr}
-           , phaseSlider{nullptr}
-           , noiseWhiteButton{nullptr}
-           , noiseBrownianButton{nullptr}
-           , envelopeAmplitudeKnob{nullptr}
-           , frequencyAmplitudeKnob{nullptr}
-           , filterTypeIsChecked{false}
+          , oscillator{osc}
+          , oscillatorCheckbox{nullptr}
+          , fmCheckbox{nullptr}
+          , filterBox{nullptr}
+          , sineButton{nullptr}
+          , squareButton{nullptr}
+          , triangleButton{nullptr}
+          , sawtoothButton{nullptr}
+          , sampleButton{nullptr}
+          , sampleBrowseButton{nullptr}
+          , phaseSlider{nullptr}
+          , noiseWhiteButton{nullptr}
+          , noiseBrownianButton{nullptr}
+          , amplitudeKnob{nullptr}
+          , frequencyKnob{nullptr}
+          , pitchShiftKnob{nullptr}
+          , filterTypeIsChecked{false}
+          , amplitudeEnvelopeBox{nullptr}
+          , oscFreqEnvelopeButton{nullptr}
+          , pitchEnvelopeButton{nullptr}
 {
         setFixedSize(224, 380);
         oscillatorCheckbox = new GeonkickButton(this);
         oscillatorCheckbox->setCheckable(true);
-        oscillatorCheckbox->setPosition(5, 0);
+        oscillatorCheckbox->setX(3);
         RK_ACT_BIND(oscillatorCheckbox, toggled,
                     RK_ACT_ARGS(bool b), oscillator, enable(b));
 
         if (oscillator->type() == Oscillator::Type::Oscillator1) {
-                oscillatorCheckbox->setFixedSize(85, 11);
-                oscillatorCheckbox->setPressedImage(RkImage(oscillatorCheckbox->size(),
-                                                            RK_IMAGE_RC(osc1_radio_button_active)));
-                oscillatorCheckbox->setUnpressedImage(RkImage(oscillatorCheckbox->size(),
-                                                              RK_IMAGE_RC(osc1_radio_button)));
+                oscillatorCheckbox->setFixedSize(69, 21);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_osc1_on)),
+                                             RkButton::State::Pressed);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_osc1_off)),
+                                             RkButton::State::Unpressed);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_osc1_hover)),
+                                             RkButton::State::UnpressedHover);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_osc1_hover)),
+                                             RkButton::State::PressedHover);
+
 
                 fmCheckbox = new GeonkickButton(this);
                 fmCheckbox->setCheckable(true);
-                fmCheckbox->setPosition(oscillatorCheckbox->x() + oscillatorCheckbox->width() + 20, 0);
-                fmCheckbox->setFixedSize(95, 11);
-                fmCheckbox->setPressedImage(RkImage(95, 11, RK_IMAGE_RC(fm_radio_active)));
-                fmCheckbox->setUnpressedImage(RkImage(95, 11, RK_IMAGE_RC(fm_radio)));
+                fmCheckbox->setPosition(oscillatorCheckbox->x() + oscillatorCheckbox->width() + 5, 0);
+                fmCheckbox->setFixedSize(76, 21);
+                fmCheckbox->setImage(RkImage(fmCheckbox->size(), RK_IMAGE_RC(fm_radio_active)),
+                                     RkButton::State::Pressed);
+                fmCheckbox->setImage(RkImage(fmCheckbox->size(), RK_IMAGE_RC(fm_radio)),
+                                     RkButton::State::Unpressed);
+                fmCheckbox->setImage(RkImage(fmCheckbox->size(), RK_IMAGE_RC(fm_radio_hover)),
+                                     RkButton::State::UnpressedHover);
+                fmCheckbox->setImage(RkImage(fmCheckbox->size(), RK_IMAGE_RC(fm_radio_hover)),
+                                     RkButton::State::PressedHover);
                 RK_ACT_BIND(fmCheckbox, toggled, RK_ACT_ARGS(bool b), oscillator, setAsFm(b));
         } else if (oscillator->type() == Oscillator::Type::Oscillator2) {
-                oscillatorCheckbox->setFixedSize(87, 11);
-                oscillatorCheckbox->setPressedImage(RkImage(oscillatorCheckbox->size(),
-                                                            RK_IMAGE_RC(osc2_radio_button_active)));
-                oscillatorCheckbox->setUnpressedImage(RkImage(oscillatorCheckbox->size(),
-                                                              RK_IMAGE_RC(osc2_radio_button)));
+                oscillatorCheckbox->setFixedSize(69, 21);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_osc2_on)),
+                                             RkButton::State::Pressed);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_osc2_off)),
+                                             RkButton::State::Unpressed);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_osc2_hover)),
+                                             RkButton::State::UnpressedHover);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_osc2_hover)),
+                                             RkButton::State::PressedHover);
         } else {
-                oscillatorCheckbox->setFixedSize(48, 11);
-                oscillatorCheckbox->setPressedImage(RkImage(oscillatorCheckbox->size(),
-                                                            RK_IMAGE_RC(noise_radio_button_active)));
-                oscillatorCheckbox->setUnpressedImage(RkImage(oscillatorCheckbox->size(),
-                                                              RK_IMAGE_RC(noise_radio_button)));
+                oscillatorCheckbox->setFixedSize(69, 21);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_noise_on)),
+                                             RkButton::State::Pressed);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_noise_off)),
+                                             RkButton::State::Unpressed);
+                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_noise_hover)),
+                                             RkButton::State::UnpressedHover);
+                                oscillatorCheckbox->setImage(RkImage(oscillatorCheckbox->size(),
+                                                     RK_IMAGE_RC(controls_noise_hover)),
+                                             RkButton::State::PressedHover);
         }
 
         if (oscillator->type() != Oscillator::Type::Noise)
@@ -128,7 +177,7 @@ void OscillatorGroupBox::createWaveFunctionGroupBox()
         auto waveFunctionHBox = new GeonkickWidget(this);
         waveFunctionHBox->setBackgroundColor(67, 68, 68);
         waveFunctionHBox->setFixedSize(224, 85);
-        waveFunctionHBox->setPosition(0, 18);
+        waveFunctionHBox->setPosition(0, 25);
         waveFunctionHBox->setBackgroundImage(RkImage(waveFunctionHBox->size(), RK_IMAGE_RC(wf_bk_hbox)));
         waveFunctionHBox->show();
 
@@ -182,7 +231,7 @@ void OscillatorGroupBox::createWaveFunctionGroupBox()
         sampleBrowseButton->setFixedSize(67, 14);
         sampleBrowseButton->setPosition(triangleButton->x() + 73, sampleButton->y() + sampleButton->height() + 5);
         sampleBrowseButton->setUnpressedImage(RkImage(sampleBrowseButton->size(), RK_IMAGE_RC(button_browse_sample)));
-        RK_ACT_BIND(sampleBrowseButton, toggled, RK_ACT_ARGS(bool b), this, browseSample());
+        RK_ACT_BIND(sampleBrowseButton, pressed, RK_ACT_ARGS(), this, browseSample());
 
         auto phaseLabel = new RkLabel(waveFunctionHBox);
         phaseLabel->setFixedSize(30, 8);
@@ -201,30 +250,56 @@ void OscillatorGroupBox::createWaveFunctionGroupBox()
 
 void OscillatorGroupBox::createEvelopeGroupBox()
 {
-        auto amplitudeEnvelopeBox = new GeonkickWidget(this);
+        amplitudeEnvelopeBox = new GeonkickWidget(this);
         amplitudeEnvelopeBox->setFixedSize(224, 125);
         if (oscillator->type() == Oscillator::Type::Noise) {
                 amplitudeEnvelopeBox->setBackgroundImage(RkImage(224, 125, RK_IMAGE_RC(hboxbk_noise_env)));
-                amplitudeEnvelopeBox->setPosition(0, 18);
+                amplitudeEnvelopeBox->setPosition(0, 25);
         } else {
                 amplitudeEnvelopeBox->setBackgroundImage(RkImage(224, 125, RK_IMAGE_RC(hboxbk_osc_env)));
-                amplitudeEnvelopeBox->setPosition(0, 111);
+                amplitudeEnvelopeBox->setPosition(0, 110);
         }
         amplitudeEnvelopeBox->show();
 
-        envelopeAmplitudeKnob = new Knob(amplitudeEnvelopeBox);
-        envelopeAmplitudeKnob->setPosition((224 / 2 - 80) / 2, (125 - 80) / 2 - 1);
-        envelopeAmplitudeKnob->setFixedSize(80, 80);
-        envelopeAmplitudeKnob->setKnobBackgroundImage(RkImage(80, 80, RK_IMAGE_RC(knob_bk_image)));
-        envelopeAmplitudeKnob->setKnobImage(RkImage(70, 70, RK_IMAGE_RC(knob)));
-        envelopeAmplitudeKnob->setRange(0, 1.0);
-        envelopeAmplitudeKnob->show();
-        RK_ACT_BIND(envelopeAmplitudeKnob,
+        amplitudeKnob = new Knob(amplitudeEnvelopeBox);
+        amplitudeKnob->setPosition((224 / 2 - 80) / 2, (125 - 80) / 2 - 1);
+        amplitudeKnob->setFixedSize(80, 80);
+        amplitudeKnob->setKnobBackgroundImage(RkImage(80, 80, RK_IMAGE_RC(knob_bk_image)));
+        amplitudeKnob->setKnobImage(RkImage(70, 70, RK_IMAGE_RC(knob)));
+        amplitudeKnob->setRange(0, 1.0);
+        amplitudeKnob->show();
+        RK_ACT_BIND(amplitudeKnob,
                     valueUpdated,
                     RK_ACT_ARGS(double val),
                     oscillator,
                     setAmplitude(val));
 
+        auto oscAmplEnvelopeButton = new GeonkickButton(amplitudeEnvelopeBox);
+        oscAmplEnvelopeButton->setPressed(viewState()->getEnvelopeType() == Envelope::Type::Amplitude
+                                          && static_cast<Oscillator::Type>(viewState()->getEnvelopeCategory())
+                                          == oscillator->type());
+        oscAmplEnvelopeButton->setFixedSize(63, 21);
+        oscAmplEnvelopeButton->setPosition(amplitudeKnob->x() + amplitudeKnob->width() / 2 - oscAmplEnvelopeButton->width() / 2,
+                                           amplitudeKnob->y() + amplitudeKnob->height() - 3);
+        oscAmplEnvelopeButton->setImage(RkImage(oscAmplEnvelopeButton->size(), RK_IMAGE_RC(osc_ampl_button_off)),
+                                        RkButton::State::Unpressed);
+        oscAmplEnvelopeButton->setImage(RkImage(oscAmplEnvelopeButton->size(), RK_IMAGE_RC(osc_ampl_button_on)),
+                                        RkButton::State::Pressed);
+        oscAmplEnvelopeButton->setImage(RkImage(oscAmplEnvelopeButton->size(), RK_IMAGE_RC(osc_ampl_button_hover)),
+                                        RkButton::State::PressedHover);
+        oscAmplEnvelopeButton->setImage(RkImage(oscAmplEnvelopeButton->size(), RK_IMAGE_RC(osc_ampl_button_hover)),
+                                        RkButton::State::UnpressedHover);
+        oscAmplEnvelopeButton->show();
+        RK_ACT_BIND(oscAmplEnvelopeButton,
+                    pressed,
+                    RK_ACT_ARGS(),
+                    viewState(), setEnvelope(static_cast<Envelope::Category>(oscillator->type()),
+                                             Envelope::Type::Amplitude));
+        RK_ACT_BIND(viewState(), envelopeChanged,
+                    RK_ACT_ARGS(Envelope::Category category, Envelope::Type envelope),
+                    oscAmplEnvelopeButton, setPressed(envelope == Envelope::Type::Amplitude
+                                                      && static_cast<Oscillator::Type>(category)
+                                                      == oscillator->type()));
         if (oscillator->type() == Oscillator::Type::Noise) {
                 noiseWhiteButton = new GeonkickButton(amplitudeEnvelopeBox);
                 noiseWhiteButton->setPosition(224 / 2 + (224 / 2 - 90) / 2 - 10, 10);
@@ -259,29 +334,99 @@ void OscillatorGroupBox::createEvelopeGroupBox()
                 RK_ACT_BIND(seedSlider, valueUpdated, RK_ACT_ARGS(int value), this, setOscillatorSeed(value));
 
         } else {
-                frequencyAmplitudeKnob = new Knob(amplitudeEnvelopeBox);
-                frequencyAmplitudeKnob->setRangeType(Knob::RangeType::Logarithmic);
-                frequencyAmplitudeKnob->setSize(80, 80);
-                frequencyAmplitudeKnob->setPosition(224 / 2 + (224 / 2 - 80) / 2, (125 - 80) / 2 - 1);
-                frequencyAmplitudeKnob->setKnobBackgroundImage(RkImage(80, 80, RK_IMAGE_RC(knob_bk_image)));
-                frequencyAmplitudeKnob->setKnobImage(RkImage(70, 70, RK_IMAGE_RC(knob)));
-                frequencyAmplitudeKnob->setRange(200, 16000);
-                RK_ACT_BIND(frequencyAmplitudeKnob,
+                pitchShiftKnob = new Knob(amplitudeEnvelopeBox);
+                pitchShiftKnob->setSize(80, 80);
+                pitchShiftKnob->setPosition(224 / 2 + (224 / 2 - 80) / 2, (125 - 80) / 2 - 1);
+                pitchShiftKnob->setKnobBackgroundImage(RkImage(80, 80, RK_IMAGE_RC(knob_bk_image)));
+                pitchShiftKnob->setKnobImage(RkImage(70, 70, RK_IMAGE_RC(knob)));
+                pitchShiftKnob->setRange(0, 48);
+                RK_ACT_BIND(pitchShiftKnob,
+                            valueUpdated,
+                            RK_ACT_ARGS(double val),
+                            oscillator,
+                            setPitchShift(val));
+                frequencyKnob = new Knob(amplitudeEnvelopeBox);
+                frequencyKnob->setRangeType(Knob::RangeType::Logarithmic);
+                frequencyKnob->setSize(80, 80);
+                frequencyKnob->setPosition(224 / 2 + (224 / 2 - 80) / 2, (125 - 80) / 2 - 1);
+                frequencyKnob->setKnobBackgroundImage(RkImage(80, 80, RK_IMAGE_RC(knob_bk_image)));
+                frequencyKnob->setKnobImage(RkImage(70, 70, RK_IMAGE_RC(knob)));
+                frequencyKnob->setRange(200, 16000);
+                RK_ACT_BIND(frequencyKnob,
                             valueUpdated,
                             RK_ACT_ARGS(double val),
                             oscillator,
                             setFrequency(val));
-                frequencyAmplitudeKnob->show();
+
+                oscFreqEnvelopeButton = new GeonkickButton(amplitudeEnvelopeBox);
+                oscFreqEnvelopeButton->setPressed(viewState()->getEnvelopeType() == Envelope::Type::Frequency
+                                                  && static_cast<Oscillator::Type>(viewState()->getEnvelopeCategory())
+                                                  == oscillator->type());
+                oscFreqEnvelopeButton->setFixedSize(63, 21);
+                oscFreqEnvelopeButton->setPosition(frequencyKnob->x() + frequencyKnob->width() / 2 - oscFreqEnvelopeButton->width() / 2,
+                                                   frequencyKnob->y() + frequencyKnob->height() - 3);
+                oscFreqEnvelopeButton->setImage(RkImage(oscFreqEnvelopeButton->size(), RK_IMAGE_RC(osc_freq_button_off)),
+                                                RkButton::State::Unpressed);
+                oscFreqEnvelopeButton->setImage(RkImage(oscFreqEnvelopeButton->size(), RK_IMAGE_RC(osc_freq_button_on)),
+                                                RkButton::State::Pressed);
+                oscFreqEnvelopeButton->setImage(RkImage(oscFreqEnvelopeButton->size(), RK_IMAGE_RC(osc_freq_button_hover)),
+                                                RkButton::State::PressedHover);
+                oscFreqEnvelopeButton->setImage(RkImage(oscFreqEnvelopeButton->size(), RK_IMAGE_RC(osc_freq_button_hover)),
+                                                RkButton::State::UnpressedHover);
+                RK_ACT_BIND(oscFreqEnvelopeButton,
+                            pressed,
+                            RK_ACT_ARGS(),
+                            viewState(), setEnvelope(static_cast<Envelope::Category>(oscillator->type()),
+                                                     Envelope::Type::Frequency));
+                RK_ACT_BIND(viewState(), envelopeChanged,
+                            RK_ACT_ARGS(Envelope::Category category, Envelope::Type envelope),
+                            oscFreqEnvelopeButton, setPressed(envelope == Envelope::Type::Frequency
+                                                              && static_cast<Oscillator::Type>(category)
+                                                              == oscillator->type()));
+
+                pitchEnvelopeButton = new GeonkickButton(amplitudeEnvelopeBox);
+                pitchEnvelopeButton->setPressed(viewState()->getEnvelopeType() == Envelope::Type::PitchShift
+                                                && static_cast<Oscillator::Type>(viewState()->getEnvelopeCategory())
+                                                == oscillator->type());
+                pitchEnvelopeButton->setFixedSize(63, 21);
+                pitchEnvelopeButton->setPosition(frequencyKnob->x() + frequencyKnob->width() / 2 - pitchEnvelopeButton->width() / 2,
+                                                 frequencyKnob->y() + frequencyKnob->height() - 3);
+                pitchEnvelopeButton->setImage(RkImage(pitchEnvelopeButton->size(), RK_IMAGE_RC(osc_pitch_button_off)),
+                                              RkButton::State::Unpressed);
+                pitchEnvelopeButton->setImage(RkImage(pitchEnvelopeButton->size(), RK_IMAGE_RC(osc_pitch_button_on)),
+                                              RkButton::State::Pressed);
+                pitchEnvelopeButton->setImage(RkImage(pitchEnvelopeButton->size(), RK_IMAGE_RC(osc_pitch_button_hover)),
+                                              RkButton::State::PressedHover);
+                pitchEnvelopeButton->setImage(RkImage(pitchEnvelopeButton->size(), RK_IMAGE_RC(osc_pitch_button_hover)),
+                                              RkButton::State::UnpressedHover);
+                RK_ACT_BIND(pitchEnvelopeButton,
+                            pressed,
+                            RK_ACT_ARGS(),
+                            viewState(), setEnvelope(static_cast<Envelope::Category>(oscillator->type()),
+                                                     Envelope::Type::PitchShift));
+                RK_ACT_BIND(viewState(), envelopeChanged,
+                            RK_ACT_ARGS(Envelope::Category category, Envelope::Type envelope),
+                            pitchEnvelopeButton, setPressed(envelope == Envelope::Type::PitchShift
+                                                            && static_cast<Oscillator::Type>(category)
+                                                            == oscillator->type()));
+
+                if (oscillator->function() == Oscillator::FunctionType::Sample) {
+                        pitchEnvelopeButton->show();
+                        pitchShiftKnob->show();
+                } else {
+                        oscFreqEnvelopeButton->show();
+                        frequencyKnob->show();
+                }
         }
 }
 
 void OscillatorGroupBox::createFilterGroupBox()
 {
-        filterBox = new Filter(this);
+        filterBox = new Filter(this, static_cast<Envelope::Category>(oscillator->type()));
         if (oscillator->type() == Oscillator::Type::Noise)
-                filterBox->setPosition(0, 151);
+                filterBox->setPosition(0, 150);
         else
-                filterBox->setPosition(0, 243);
+                filterBox->setPosition(0, 235);
         filterBox->setCutOffRange(20, 20000);
         filterBox->setResonanceRange(1, 1000);
         RK_ACT_BIND(filterBox, enabled, RK_ACT_ARGS(bool b),
@@ -302,6 +447,7 @@ void OscillatorGroupBox::setSineWave(bool pressed)
                 sawtoothButton->setPressed(false);
                 sampleButton->setPressed(false);
                 oscillator->setFunction(Oscillator::FunctionType::Sine);
+                updateAmpltudeEnvelopeBox();
         }
 }
 
@@ -313,6 +459,7 @@ void OscillatorGroupBox::setSquareWave(bool pressed)
                 sawtoothButton->setPressed(false);
                 sampleButton->setPressed(false);
                 oscillator->setFunction(Oscillator::FunctionType::Square);
+                updateAmpltudeEnvelopeBox();
         }
 }
 
@@ -324,6 +471,7 @@ void OscillatorGroupBox::setTriangleWave(bool pressed)
                 sawtoothButton->setPressed(false);
                 sampleButton->setPressed(false);
                 oscillator->setFunction(Oscillator::FunctionType::Triangle);
+                updateAmpltudeEnvelopeBox();
         }
 }
 
@@ -335,6 +483,7 @@ void OscillatorGroupBox::setSawtoothWave(bool pressed)
                 triangleButton->setPressed(false);
                 sampleButton->setPressed(false);
                 oscillator->setFunction(Oscillator::FunctionType::Sawtooth);
+                updateAmpltudeEnvelopeBox();
         }
 }
 
@@ -346,6 +495,7 @@ void OscillatorGroupBox::setSampleFunction(bool pressed)
                 triangleButton->setPressed(false);
                 sawtoothButton->setPressed(false);
                 oscillator->setFunction(Oscillator::FunctionType::Sample);
+                updateAmpltudeEnvelopeBox();
         }
 }
 
@@ -400,11 +550,14 @@ void OscillatorGroupBox::updateGui()
                 sawtoothButton->setPressed(oscillator->function() == Oscillator::FunctionType::Sawtooth);
                 sampleButton->setPressed(oscillator->function() == Oscillator::FunctionType::Sample);
                 phaseSlider->onSetValue(oscillator->getPhase());
+                updateAmpltudeEnvelopeBox();
         }
 
-        envelopeAmplitudeKnob->setCurrentValue(oscillator->amplitude());
-        if (oscillator->type() != Oscillator::Type::Noise)
-                frequencyAmplitudeKnob->setCurrentValue(oscillator->frequency());
+        amplitudeKnob->setCurrentValue(oscillator->amplitude());
+        if (oscillator->type() != Oscillator::Type::Noise) {
+                frequencyKnob->setCurrentValue(oscillator->frequency());
+                pitchShiftKnob->setCurrentValue(oscillator->pitchShift());
+        }
 
         if (oscillator->type() == Oscillator::Type::Oscillator1)
                 fmCheckbox->setPressed(oscillator->isFm());
@@ -417,10 +570,15 @@ void OscillatorGroupBox::updateGui()
 
 void OscillatorGroupBox::browseSample()
 {
-        auto fileDialog = new FileDialog(this, FileDialog::Type::Open, "Select sample");
-        fileDialog->setFilters({".wav", ".WAV", ".flac", ".FLAC", ".ogg", ".OGG"});
-        fileDialog->setCurrentDirectoy(oscillator->samplesPath());
-        RK_ACT_BIND(fileDialog, selectedFile,
-                    RK_ACT_ARGS(const std::string &file),
-                    oscillator, setSample(file));
+        viewState()->setSamplesBrowserOscillator(static_cast<ViewState::Oscillator>(oscillator->type()));
+        viewState()->setMainView(ViewState::View::Samples);
+}
+
+void OscillatorGroupBox::updateAmpltudeEnvelopeBox()
+{
+        pitchShiftKnob->show(oscillator->function() == Oscillator::FunctionType::Sample);
+        pitchEnvelopeButton->show(oscillator->function() == Oscillator::FunctionType::Sample);
+        frequencyKnob->show(oscillator->function() != Oscillator::FunctionType::Sample);
+        oscFreqEnvelopeButton->show(oscillator->function() != Oscillator::FunctionType::Sample);
+        amplitudeEnvelopeBox->update();
 }
