@@ -2,7 +2,7 @@
  * File name: percussion_state.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2018 Iuri Nistor <http://iuriepage.wordpress.com>
+ * Copyright (C) 2018 Iuri Nistor 
  *
  * This file is part of Geonkick.
  *
@@ -22,7 +22,7 @@
  */
 
 #include "percussion_state.h"
-#include "base64.h"
+#include "Base64EncoderDecoder.h"
 
 #include <iomanip>
 
@@ -70,21 +70,28 @@ bool PercussionState::loadFile(const std::string &file)
         std::ifstream sfile;
         sfile.open(std::filesystem::absolute(filePath));
         if (!sfile.is_open()) {
-                RK_LOG_ERROR("can't open preset.");
+	        RK_LOG_ERROR("can't open preset file " << file);
                 return false;
         }
 
         std::string fileData((std::istreambuf_iterator<char>(sfile)),
                              (std::istreambuf_iterator<char>()));
-        loadData(fileData);
+        if (!loadData(fileData)) {
+	        GEONKICK_LOG_ERROR("error on parsing preset file " << file);
+	        return false;
+        }
         return true;
 }
 
-void PercussionState::loadData(const std::string &data)
+bool PercussionState::loadData(const std::string &data)
 {
         rapidjson::Document document;
-        document.Parse(data.c_str());
+        if (document.Parse(data.c_str()).HasParseError()) {
+	        GEONKICK_LOG_ERROR("error on parsing JSON data");
+	        return false;
+        }
         loadObject(document);
+	return true;
 }
 
 void PercussionState::loadObject(const rapidjson::Value &obj)
@@ -359,6 +366,16 @@ void PercussionState::parseOscillatorObject(int index,  const rapidjson::Value &
                                                                             GeonkickApi::EnvelopeType::Frequency);
                                 }
                         }
+
+                        if (m.name == "pitchshift_env" && m.value.IsObject()) {
+                                for (const auto &el: m.value.GetObject()) {
+                                        if (el.name == "amplitude" && el.value.IsDouble())
+                                                setOscillatorPitchShift(index, el.value.GetDouble());
+                                        if (el.name == "points" && el.value.IsArray())
+                                                setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
+                                                                            GeonkickApi::EnvelopeType::PitchShift);
+                                }
+                        }
                 }
 
                 if (m.name == "filter" && m.value.IsObject()) {
@@ -543,6 +560,12 @@ void PercussionState::setOscillatorFrequency(int index, double val)
                 oscillator->frequency = val;
 }
 
+void PercussionState::setOscillatorPitchShift(int index, double val)
+{
+        if (auto oscillator = getOscillator(index); oscillator)
+                oscillator->pitchShift = val;
+}
+
 void PercussionState::setOscillatorFilterEnabled(int index, bool b)
 {
         auto oscillator = getOscillator(index);
@@ -577,12 +600,22 @@ void PercussionState::setOscillatorEnvelopePoints(int index,
 {
         auto oscillator = getOscillator(index);
         if (oscillator) {
-                if (envelope == GeonkickApi::EnvelopeType::Amplitude)
+                switch (envelope) {
+                case GeonkickApi::EnvelopeType::Amplitude:
                         oscillator->amplitudeEnvelope = points;
-                else if (envelope == GeonkickApi::EnvelopeType::Frequency)
+                        break;
+                case GeonkickApi::EnvelopeType::Frequency:
                         oscillator->frequencyEnvelope = points;
-                else
+                        break;
+                case GeonkickApi::EnvelopeType::PitchShift:
+                        oscillator->pitchShiftEnvelope = points;
+                        break;
+                case GeonkickApi::EnvelopeType::FilterCutOff:
                         oscillator->filterCutOffEnvelope = points;
+                        break;
+                default:
+                        return;
+                }
         }
 }
 
@@ -639,24 +672,28 @@ int PercussionState::oscillatorSeed(int index) const
 
 double PercussionState::oscillatorAmplitue(int index) const
 {
-        auto oscillator = getOscillator(index);
-        if (oscillator)
+        if (auto oscillator = getOscillator(index); oscillator)
                 return oscillator->amplitude;
         return 0;
 }
 
 double PercussionState::oscillatorFrequency(int index) const
 {
-        auto oscillator = getOscillator(index);
-        if (oscillator)
+        if (auto oscillator = getOscillator(index); oscillator)
                 return oscillator->frequency;
         return 0;
 }
 
+double PercussionState::oscillatorPitchShift(int index) const
+{
+        if (auto oscillator = getOscillator(index); oscillator)
+                return oscillator->pitchShift;
+        return 0;
+}
+
 bool PercussionState::isOscillatorFilterEnabled(int index) const
 {
-        auto oscillator = getOscillator(index);
-        if (oscillator)
+        if (auto oscillator = getOscillator(index); oscillator)
                 return oscillator->isFilterEnabled;
         return false;
 }
@@ -688,16 +725,20 @@ double PercussionState::oscillatorFilterFactor(int index) const
 std::vector<RkRealPoint>
 PercussionState::oscillatorEnvelopePoints(int index, GeonkickApi::EnvelopeType type) const
 {
-        auto oscillator = getOscillator(index);
-        if (oscillator) {
-                if (type == GeonkickApi::EnvelopeType::Amplitude)
+        if (auto oscillator = getOscillator(index); oscillator) {
+                switch (type) {
+                case GeonkickApi::EnvelopeType::Amplitude:
                         return oscillator->amplitudeEnvelope;
-                else if (type == GeonkickApi::EnvelopeType::Frequency)
+                case GeonkickApi::EnvelopeType::Frequency:
                         return oscillator->frequencyEnvelope;
-                else
+                case GeonkickApi::EnvelopeType::PitchShift:
+                        return oscillator->pitchShiftEnvelope;
+                case GeonkickApi::EnvelopeType::FilterCutOff:
                         return oscillator->filterCutOffEnvelope;
+                default:
+                        return {};
+                }
         }
-
         return {};
 }
 
@@ -814,6 +855,7 @@ double PercussionState::getDistortionDrive() const
 std::string PercussionState::toJson() const
 {
         std::ostringstream jsonStream;
+        jsonStream << std::fixed << std::setprecision(7);
         jsonStream << "{" << std::endl;
         oscJson(jsonStream);
         kickJson(jsonStream);
@@ -821,6 +863,26 @@ std::string PercussionState::toJson() const
         return jsonStream.str();
 }
 
+void PercussionState::envelopeToJson(std::ostringstream &jsonStream,
+                                     const std::string &envName,
+                                     double amplitude,
+                                     const std::vector<RkRealPoint> &envelope)
+{
+        jsonStream << "\"" << envName << "\": {" << std::endl;
+        jsonStream << "\"amplitude\": " << amplitude << ", " << std::endl;
+        jsonStream << "\"points\": [" << std::endl;
+        bool first = true;
+        for (const auto &point: envelope) {
+                if (!first)
+                        jsonStream << ", ";
+                jsonStream << "[ "  << point.x()
+                           << " , "  << point.y() << "]";
+                first = false;
+        }
+        jsonStream << "]" << std::endl;
+        jsonStream << "}" << std::endl;
+}
+
 void PercussionState::oscJson(std::ostringstream &jsonStream) const
 {
         for (const auto& val: oscillators) {
@@ -830,57 +892,42 @@ void PercussionState::oscJson(std::ostringstream &jsonStream) const
                 if (val.second->function == GeonkickApi::FunctionType::Sample && !val.second->sample.empty())
                         jsonStream <<  "\"sample\": \"" << toBase64F(val.second->sample) << "\"," << std::endl;
                 jsonStream <<  "\"function\": " << static_cast<int>(val.second->function) << "," << std::endl;
-                jsonStream <<  "\"phase\": " << std::fixed << std::setprecision(5)
+                jsonStream <<  "\"phase\": " 
                            << val.second->phase << ", " << std::endl;
                 jsonStream <<  "\"seed\": " << val.second->seed << ", " << std::endl;
-                jsonStream << "\"ampl_env\": {" << std::endl;
-                jsonStream << "\"amplitude\": "  << std::fixed << std::setprecision(5)
-                           << val.second->amplitude << ", " << std::endl;
-                jsonStream << "\"points\": [" << std::endl;
-                bool first = true;
-                for (const auto &point: val.second->amplitudeEnvelope) {
-                        if (first)
-                                first = false;
-                        else
-                                jsonStream << ", ";
-                        jsonStream << "[ " << std::fixed << std::setprecision(5) << point.x()
-                                   << " , " << std::fixed << std::setprecision(5) << point.y() << "]";
-                }
-                jsonStream << "]" << std::endl; // points
-                jsonStream << "}," << std::endl; // ampl_env
-
-                jsonStream << "\"freq_env\": {" << std::endl;
-                jsonStream << "\"amplitude\": " << val.second->frequency << ", " << std::endl;
-                jsonStream << "\"points\": [" << std::endl;
-                first = true;
-                for (const auto &point: val.second->frequencyEnvelope) {
-                        if (first)
-                                first = false;
-                        else
-                                jsonStream << ", ";
-                        jsonStream << "[ " << std::fixed << std::setprecision(5) << point.x()
-                                   << " , " << std::fixed << std::setprecision(5) << point.y() << "]";
-                }
-                jsonStream << "]" << std::endl; // points
-                jsonStream << "}," << std::endl; // freq_env
+                envelopeToJson(jsonStream,
+                               "ampl_env",
+                               val.second->amplitude,
+                               val.second->amplitudeEnvelope);
+                jsonStream << "," << std::endl;
+                envelopeToJson(jsonStream,
+                               "freq_env",
+                               val.second->frequency,
+                               val.second->frequencyEnvelope);
+                jsonStream << "," << std::endl;
+                envelopeToJson(jsonStream,
+                               "pitchshift_env",
+                               val.second->pitchShift,
+                               val.second->pitchShiftEnvelope);
+                jsonStream << "," << std::endl;
                 jsonStream << "\"filter\": {" << std::endl;
                 jsonStream << "\"enabled\": " << (val.second->isFilterEnabled ? "true" : "false");
                 jsonStream << ", " << std::endl;
                 jsonStream << "\"type\": " << static_cast<int>(val.second->filterType) << ", " << std::endl;
-                jsonStream << "\"cutoff\": " << std::fixed << std::setprecision(5)
+                jsonStream << "\"cutoff\": " 
                            << val.second->filterFrequency << ", " << std::endl;
                 jsonStream << "\"cutoff_env\": [";
-                first = true;
+                bool first = true;
                 for (const auto &point: val.second->filterCutOffEnvelope) {
                         if (first)
                                 first = false;
                         else
                                 jsonStream << ", ";
-                        jsonStream << "[ " << std::fixed << std::setprecision(5) << point.x()
-                                   << " , " << std::fixed << std::setprecision(5) << point.y() << "]";
+                        jsonStream << "[ "  << point.x()
+                                   << " , "  << point.y() << "]";
                 }
                 jsonStream << "], " << std::endl;
-                jsonStream << "\"factor\": " << std::fixed << std::setprecision(5)
+                jsonStream << "\"factor\": " 
                            << val.second->filterFactor << std::endl;
                 jsonStream << "}" << std::endl;  // filter;
                 jsonStream << "}" << std::endl;  // osc;
@@ -918,15 +965,15 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
                                 first = false;
                         else
                                 jsonStream << ", ";
-                        jsonStream << std::fixed << std::setprecision(5) << layersAmplitude[i];
+                        jsonStream  << layersAmplitude[i];
         }
         jsonStream << "]," << std::endl;
-        jsonStream << "\"limiter\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"limiter\": " 
                    << getLimiterValue() << ", " << std::endl;
         jsonStream << "\"tuned_output\": " << (isOutputTuned() ? "true" : "false") << ", " << std::endl;
         jsonStream << "\"ampl_env\": {" << std::endl;
-        jsonStream << "\"amplitude\": " << static_cast<double>(getKickAmplitude()) << ", " << std::endl;
-        jsonStream << "\"length\": " << static_cast<double>(getKickLength()) << ", " << std::endl;
+        jsonStream << "\"amplitude\": " << getKickAmplitude() << ", " << std::endl;
+        jsonStream << "\"length\": " << getKickLength() << ", " << std::endl;
         auto points = getKickEnvelopePoints(GeonkickApi::EnvelopeType::Amplitude);
         jsonStream << "\"points\": [";
         first = true;
@@ -935,8 +982,8 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
                         first = false;
                 else
                         jsonStream << ", ";
-                jsonStream << "[ " << std::fixed << std::setprecision(5) << point.x()
-                           << " , " << std::fixed << std::setprecision(5) << point.y() << "]";
+                jsonStream << "[ "  << point.x()
+                           << " , "  << point.y() << "]";
         }
 
         jsonStream << "]}, " << std::endl;
@@ -945,9 +992,9 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
         jsonStream << "\"enabled\": " << (isKickFilterEnabled() ? "true" : "false");
         jsonStream << ", " << std::endl;
         jsonStream << "\"type\": " << static_cast<int>(getKickFilterType()) << ", " << std::endl;
-        jsonStream << "\"cutoff\": " << std::fixed << std::setprecision(2)
+        jsonStream << "\"cutoff\": "
                    << getKickFilterFrequency() << ", " << std::endl;
-        jsonStream << "\"factor\": " << std::fixed << std::setprecision(2)
+        jsonStream << "\"factor\": "
                    << getKickFilterQFactor() << ", " << std::endl;
         points = getKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff);
         jsonStream << "\"cutoff_env\": [";
@@ -957,34 +1004,34 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
                         first = false;
                 else
                         jsonStream << ", ";
-                jsonStream << "[ " << std::fixed << std::setprecision(5) << point.x()
-                           << " , " << std::fixed << std::setprecision(5) << point.y() << "]";
+                jsonStream << "[ "  << point.x()
+                           << " , "  << point.y() << "]";
         }
         jsonStream << "]" << std::endl; // points
         jsonStream << "}, " << std::endl;  // filter;
         jsonStream << "\"compressor\": {" << std::endl;
         jsonStream << "\"enabled\": " << (isCompressorEnabled() ? "true" : "false") << ", " << std::endl;
-        jsonStream << "\"attack\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"attack\": " 
                    << getCompressorAttack() << ", " << std::endl;
-        jsonStream << "\"release\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"release\": " 
                    << getCompressorRelease() << ", " << std::endl;
-        jsonStream << "\"threshold\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"threshold\": " 
                    << getCompressorThreshold() << ", " << std::endl;
-        jsonStream << "\"ratio\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"ratio\": " 
                    << getCompressorRatio() << ", " << std::endl;
-        jsonStream << "\"knee\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"knee\": " 
                    << getCompressorKnee() << ", " << std::endl;
-        jsonStream << "\"makeup\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"makeup\": " 
                    << getCompressorMakeup() << std::endl;
         jsonStream << "}, " << std::endl;
 
         jsonStream << "\"distortion\": {" << std::endl;
         jsonStream << "\"enabled\": " << (isDistortionEnabled() ? "true" : "false") << ", " << std::endl;
-        jsonStream << "\"in_limiter\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"in_limiter\": " 
                    << getDistortionInLimiter()  << ", " << std::endl;
-        jsonStream << "\"volume\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"volume\": " 
                    << getDistortionVolume()  << ", " << std::endl;
-        jsonStream << "\"drive\": " << std::fixed << std::setprecision(5)
+        jsonStream << "\"drive\": " 
                    << getDistortionDrive() << ", " << std::endl;
 	jsonStream << "\"drive_env\": [" << std::endl;
 	points = getKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionDrive);
@@ -994,8 +1041,8 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
                         first = false;
                 else
                         jsonStream << ", ";
-                jsonStream << "[ " << std::fixed << std::setprecision(5) << point.x()
-                           << " , " << std::fixed << std::setprecision(5) << point.y() << "]";
+                jsonStream << "[ "  << point.x()
+                           << " , "  << point.y() << "]";
         }
         jsonStream << "], " << std::endl;
         jsonStream << "\"volume_env\": [" << std::endl;
@@ -1006,8 +1053,8 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
                         first = false;
                 else
                         jsonStream << ", ";
-                jsonStream << "[ " << std::fixed << std::setprecision(5) << point.x()
-                           << " , " << std::fixed << std::setprecision(5) << point.y() << "]";
+                jsonStream << "[ "  << point.x()
+                           << " , " << point.y() << "]";
         }
 	jsonStream << "]" << std::endl;
         jsonStream << "}" << std::endl; // distortion
@@ -1062,31 +1109,12 @@ bool PercussionState::isOutputTuned() const
 
 std::vector<float> PercussionState::fromBase64F(const std::string &str)
 {
-        size_t len;
-        auto data_str = base64_decode(reinterpret_cast<const unsigned char*>(str.c_str()),
-                                      str.size(),
-                                      &len);
-        if (data_str && len > sizeof(float)) {
-                std::vector<float> data(reinterpret_cast<float*>(data_str),
-                                        reinterpret_cast<float*>(data_str) + len / sizeof(float));
-                free(data_str);
-                return data;
-        }
-        return {};
+        return Base64EncoderDecoder::decode(str);
 }
 
 std::string PercussionState::toBase64F(const std::vector<float> &data)
 {
-        size_t len;
-        auto base64 = base64_encode(reinterpret_cast<const unsigned char*>(data.data()),
-                                    data.size() * sizeof(float), &len);
-        if (base64  && len > 0) {
-                std::string str(reinterpret_cast<const char*>(base64), len);
-                free(base64);
-                str.erase(std::remove(str.begin(), str.end(), '\n'), str.end());
-                return str;
-        }
-        return {};
+        return Base64EncoderDecoder::encode(data);
 }
 
 bool PercussionState::save(const std::string &fileName)
